@@ -1,5 +1,5 @@
 # prime/engine/engine-basic.rb
-# $Id: engine-userdict.rb,v 1.8.2.8 2004/01/15 02:38:36 komatsu Exp $
+# $Id: engine-userdict.rb,v 1.8.2.14 2004/01/28 08:34:56 komatsu Exp $
 #
 # Copyright (C) 2002 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -11,7 +11,7 @@
 
 require 'sary'
 require 'prime/prime-dict-config.rb'
-require 'prime/engine/engine-japanese'
+require 'prime/engine/engine'
 
 USERDICT = 'userdict'
 $engine_class_name = 'PrimeEngineUserdict'
@@ -98,7 +98,7 @@ class PrimeUserDict < Sary::Searcher
 end
 
 
-class PrimeEngineUserdict < PrimeEngineJapanese
+class PrimeEngineUserdict < PrimeEngine
   def initialize
     super
 
@@ -173,6 +173,85 @@ class PrimeEngineUserdict < PrimeEngineJapanese
 
   # ---------------------------------------- End of intialization.
 
+  def search (query)
+    if query.input == [""] then
+      return PrimeWordList.new()
+    end
+
+    word_lines_with_context = []
+    case query.method
+    when :prefix then
+      key = query.input
+      max   = 10
+      if query.context.non_nil? then
+        (indexes, word_lines_with_context) =
+            lookup_dict_with_context(key, query.context, max)
+        max -= word_lines_with_context.length
+      end
+      word_lines = lookup_dict(key, max)
+    when :exact then
+      query_format = (query.pos ? "%s\t#{query.pos}\t" : "%s\t")
+      key = query.input.map{|input|
+        format(query_format, input)
+      }
+      max   = nil
+      if query.context.non_nil? then
+        (indexes, word_lines_with_context) =
+            lookup_dict_with_context(key, query.context, max)
+      end
+      word_lines = lookup_dict(key, max)
+    when :literal_prefix then
+      key = query.input
+      max = 10
+
+      word_lines = lookup_dict(key, max, @dict_literal)
+    when :literal_exact then
+      query_format = "%s\t"
+      key = query.input.map{|input|
+        format(query_format, input)
+      }
+      max   = nil
+      word_lines = lookup_dict(key, max, @dict_literal)
+    else
+      return PrimeWordList.new()
+    end
+
+    word_list = PrimeWordList.new()
+    word_lines_with_context.each {|word_line|
+      word = parse_word(word_line, 18000)
+      word_list.push(word)
+    }
+    word_lines.each {|word_line|
+      word = parse_word(word_line, 15000)
+      word_list.push(word)
+    }
+    return word_list
+  end
+
+  def parse_word(line, score_offset = 15000)
+    (index_no, pron, pos, literal, *data_list) = line.split(/\t/)
+    timestamp = lookup_timestamp(index_no.to_i)
+    score = get_priority(timestamp) + score_offset
+    data = {}
+    data_list.each {|item|
+      (key, value) = item.split(/=/)
+      data[key] = value
+    }
+    word = PrimeWord.new(pron, literal, pos, score,
+                         data['category'], data['annotation'], data_list)
+  end
+
+  def check_existence (pron, literal, pos)
+    query = [pron, pos, literal].join("\t")
+    if @dict.search(query) then
+      return true
+    else
+      return false
+    end
+  end
+
+  # ---------------------------------------- End of searching methods.
+
   def refresh ()
     initialize_userdict()
     initialize_timestamp()
@@ -317,6 +396,20 @@ class PrimeEngineUserdict < PrimeEngineJapanese
   end
 
   # ---------------------------------------- End of learning module
+
+  def get_pos_data (string)
+    pos_data = {}
+    string.increase {|pron|
+      if @partdict.search(pron + "\t") then
+        (_pron, *pos_list) = @partdict.get_next_context_line.chomp.split(/\t/)
+        pos_data[pron] = pos_list
+      end
+    }
+    return pos_data
+  end
+
+  # ---------------------------------------- End of getting a pos_list module
+
   def dictbuffer_flush ()
     if $PRIME_NO_SAVE then
       return
@@ -352,56 +445,6 @@ class PrimeEngineUserdict < PrimeEngineJapanese
   # ---------------------------------------- End of writing dictionaries.
 
   private
-  def lookup (input, method = :prefix)
-    results = PrimeResult.new
-    queries = make_queries(input.base)
-    if method == :prefix then
-      input.expands.each {|string|
-	queries.add(string) 
-      }
-      rests = queries.rests
-      max   = @max_candidates
-    else
-      queries.add(input.original, nil, "", "", false)
-      rests = [""]
-      max   = 100
-    end
-
-    rests.each {|rest|
-      query_lines = queries.query_lines(rest)
-
-      (indexes, result_lines) =
-	lookup_dict_with_context(query_lines, input.context, max)
-      result_lines.each {|result_line|
-	(index_no, pattern, part, word, freq) = result_line.split(/\t/)
-	suffix = get_suffix(input.base, pattern, rest)
-	timestamp = lookup_timestamp(index_no.to_i) 
-	priority  = 18000 + get_priority(timestamp) - (rest.length * 1000)
-	cand = PrimeCandidate.new(pattern, word, priority,
-				  part, suffix, rest)
-	results << cand
-      }
-      max -= results.length
-      (max > 0) or break
-
-      result_lines = lookup_dict(query_lines, max)
-      result_lines.each {|result_line|
-	(index_no, pattern, part, word, freq) = result_line.split(/\t/)
-	unless indexes.member?(index_no) then
-	  suffix = get_suffix(input.base, pattern, rest)
-	  timestamp = lookup_timestamp(index_no.to_i) 
-	  priority  = 15000 + get_priority(timestamp) - (rest.length * 1000)
-	  cand = PrimeCandidate.new(pattern, word, priority,
-				    part, suffix, rest)
-	  results << cand
-	end
-      }
-      max -= results.length
-      (max > 0) or break
-    }
-    return results
-  end
-
   def lookup_dict_with_context (query_lines, context, max = nil)
     ## FIXME: Refact the code.
     ## FIXME: <komatsu@taiyaki.org> (2003-12-03)
@@ -420,7 +463,7 @@ class PrimeEngineUserdict < PrimeEngineJapanese
       }
     end
 
-    return [results, indexes] if max == (nil or 0)
+    return [results, indexes] if (max.non_nil? and max <= 0)
     
     if !(query_lines_with_context.empty?) and
 	@contextdict.multi_search(query_lines_with_context) then
@@ -445,13 +488,13 @@ class PrimeEngineUserdict < PrimeEngineJapanese
     return [indexes, results]
   end
 
-  def lookup_dict (query_lines, max = nil)
+  def lookup_dict (query_lines, max = nil, dict = @dict)
     results = []
-    return results if max == (nil or 0)
+    return results if (max.non_nil? and max <= 0)
 
-    if !(query_lines.empty?) and @dict.multi_search(query_lines) then
-      @dict.sort_occurrences()
-      while line = @dict.get_next_context_line do
+    if !(query_lines.empty?) and dict.multi_search(query_lines) then
+      dict.sort_occurrences()
+      while line = dict.get_next_context_line do
 	results << line.chomp
 	## FIXME: The candidates are not ordered by their priorities.
 	## FIXME: <komatsu@taiyaki.org> (2003-12-03)
@@ -467,15 +510,6 @@ class PrimeEngineUserdict < PrimeEngineJapanese
     return @timestamps[index_no]
   end
 
-  def lookup_part (base)
-    if @partdict.search(base + "\t") then
-      (yomi, *parts) = @partdict.get_next_context_line.chomp.split(/\t/)
-      return parts
-    else
-      return []
-    end
-  end
-
   def get_priority (timestamp)
     current_time = Time.new().to_i
     time_diff = current_time - timestamp
