@@ -1,5 +1,5 @@
 # session.rb
-# $Id: session.rb,v 1.1.1.1.2.1 2003/12/29 09:50:35 komatsu Exp $
+# $Id: session.rb,v 1.1.1.1.2.2 2004/01/02 04:23:07 komatsu Exp $
 #
 # Copyright (C) 2001 Satoru Takabayashi <satoru@namazu.org>
 # Copyright (C) 2002 Hiroyuki Komatsu <komatsu@taiyaki.org>
@@ -158,7 +158,7 @@ module PogemoServer
     end
 
     def accept
-      while true
+      loop {
 	socket = @server.accept
 	t = Thread.new {
 	  s = socket
@@ -169,7 +169,7 @@ module PogemoServer
 	  session.start
 	}
 	t.abort_on_exception = true
-      end
+      }
     end
   end
 
@@ -180,7 +180,7 @@ module PogemoServer
     end
 
     def accept
-      while true
+      loop {
 	socket = @server.accept
 	t = Thread.new {
 	  s = socket
@@ -191,7 +191,7 @@ module PogemoServer
 	  session.start
 	}
 	t.abort_on_exception = true
-      end
+      }
     end
   end
 
