@@ -1,5 +1,5 @@
 # taiyaki.rb: Ruby Libraries by Hiroyuki Komatsu
-# $Id: taiyaki.rb,v 1.2.2.4 2004/01/12 23:36:23 komatsu Exp $
+# $Id: taiyaki.rb,v 1.2.2.10 2004/01/28 08:34:56 komatsu Exp $
 #
 # Copyright (C) 2002 Hiroyuki Komatsu <komatsu@taiyaki.org>
 #     All rights reserved.
@@ -84,6 +84,15 @@ module Kernel
   end
 end
 
+class NilClass
+  def each
+    return nil
+  end
+
+  def empty?
+    return true
+  end
+end
 
 class String
   # '-'
@@ -124,19 +133,36 @@ class String
   end
 
   def increase
+    increasing_list = get_increasing_list()
+    if block_given? then
+      increasing_list.each {|string|
+	yield string
+      }
+    else
+      return increasing_list
+    end
+  end
+
+  def decrease
+    decreasing_list = get_increasing_list().reverse()
+    if block_given? then
+      decreasing_list.each {|string|
+	yield string
+      }
+    else
+      return decreasing_list
+    end
+  end
+
+  private
+  def get_increasing_list ()
     increasing_string = ""
     increasing_list = []
     self.split(//).each {|char|
       increasing_string += char
       increasing_list.push(increasing_string)
     }
-    if block_given? then
-      increasing_list.each {|string|
-	yield string
-      }
-    else
-      increasing_list
-    end
+    return increasing_list
   end
 end
 
@@ -189,6 +215,15 @@ class Dir
   end
 end
 
+class Hash
+  def list_push (key, list_item)
+    if self.key?(key) then
+      self[key].push(list_item)
+    else
+      self[key] = [list_item]
+    end
+  end
+end
 
 module Debug
   attr_accessor :debug_mode
@@ -207,11 +242,13 @@ end
 
 module Marshal
   def Marshal::init_file (filename, threshold_time = nil)
+    object = nil
     if File::exist?(filename) && 
 	(threshold_time.nil? || File::mtime(filename) > threshold_time) then
       object = Marshal::load_file(filename)
       Debug::message("Marshal::init_file: Load the ruby object `#{filename}'.")
-    else
+    end
+    unless object then
       object = yield
       Marshal::dump_file(object, filename)
       Debug::message("Marshal::init_file: Initialize renewally `#{filename}'.")
@@ -219,11 +256,18 @@ module Marshal
     return object
   end
 
+  ## FIXME: I'm not sure why an error happens. (?_?)
+  ## FIXME: <komatsu@taiyaki.org> (2004-01-28)
   def Marshal::load_file (filename)
-    object_file = File::open(filename, "r")
-    object = Marshal::load(object_file)
-    object_file.close
-    return object
+    begin
+      object_file = File::open(filename, "r")
+      object = Marshal::load(object_file)
+      object_file.close
+      return object
+    rescue
+      $stderr.puts "WARINIG: Marshal::load_file (#{filename}) failed."
+      return nil
+    end
   end
 
   def Marshal::dump_file (object, filename)
@@ -302,7 +346,11 @@ module Enumerable
       return sorted
     end
 
-    estimated_length = (1.4 * self.length * Math.log(self.length)).to_i
+    if self.length == 0 then
+      estimated_length = 0
+    else
+      estimated_length = (1.4 * self.length * Math.log(self.length)).to_i
+    end
     pbar = ProgressBar.new(title, estimated_length)
     if block_given? then
       sorted = self.sort {|value1, value2|
